"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.WIZARD_STORE_NAMESPACE = void 0;
var _set = _interopRequireDefault(require("lodash/set"));
var _get = _interopRequireDefault(require("lodash/get"));
var _isEmpty = _interopRequireDefault(require("lodash/isEmpty"));
var _apiFetch = _interopRequireDefault(require("@wordpress/api-fetch"));
var _data = require("@wordpress/data");
var _utils = require("./utils.js");
/**
 * External dependencies.
 */

/**
 * WordPress dependencies.
 */

const WIZARD_STORE_NAMESPACE = 'newspack/wizards';
exports.WIZARD_STORE_NAMESPACE = WIZARD_STORE_NAMESPACE;
const DEFAULT_STATE = {
  isLoading: true,
  isQuietLoading: false,
  apiData: {},
  error: null
};

/**
 * wordpress/data does not trigger a component re-render
 * on deep state change (via lodash's set function)
 * unless the state was cloned first.
 */
const clone = objectToClone => JSON.parse(JSON.stringify(objectToClone));
const reducer = function () {
  let state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_STATE;
  let {
    type,
    payload = {}
  } = arguments.length > 1 ? arguments[1] : undefined;
  switch (type) {
    case 'START_LOADING_DATA':
      if (payload.isQuietLoading) {
        return {
          ...state,
          isQuietLoading: true
        };
      }
      return {
        ...state,
        isLoading: true
      };
    case 'FINISH_LOADING_DATA':
      return {
        ...state,
        isLoading: false,
        isQuietLoading: false
      };
    case 'SET_API_DATA':
      return (0, _set.default)(clone(state), ['apiData', payload.slug], payload.data);
    case 'UPDATE_WIZARD_SETTINGS':
      return (0, _set.default)(clone(state), ['apiData', payload.slug, ...payload.path], payload.value);
    case 'SET_ERROR':
      return {
        ...state,
        error: payload
      };
    default:
      return state;
  }
};
const actions = {
  // Regular actions.
  startLoadingData: (0, _utils.createAction)('START_LOADING_DATA'),
  finishLoadingData: (0, _utils.createAction)('FINISH_LOADING_DATA'),
  fetchFromAPI: (0, _utils.createAction)('FETCH_FROM_API'),
  setAPIDataForWizard: (0, _utils.createAction)('SET_API_DATA'),
  updateWizardSettings: (0, _utils.createAction)('UPDATE_WIZARD_SETTINGS'),
  setError: (0, _utils.createAction)('SET_ERROR'),
  // Async actions. These will not show up in Redux devtools.
  saveWizardSettings(_ref) {
    let {
      slug,
      section = '',
      payloadPath = false,
      updatePayload = null
    } = _ref;
    return function* () {
      // Optionally data can be updated before saving - an immediate update case
      // (without an explicit "save" action).
      if (updatePayload) {
        yield actions.updateWizardSettings({
          slug,
          ...updatePayload
        });
      }
      const wizardState = (0, _data.select)(WIZARD_STORE_NAMESPACE).getWizardAPIData(slug);
      const updatedData = yield actions.fetchFromAPI({
        path: `/newspack/v1/wizard/${slug}/${section}`,
        method: 'POST',
        data: payloadPath ? (0, _get.default)(wizardState, payloadPath) : wizardState,
        isQuietFetch: true
      });
      if (!(0, _isEmpty.default)(updatedData) && !updatedData.error) {
        return actions.setAPIDataForWizard({
          slug,
          data: updatedData
        });
      }
    }();
  },
  *wizardApiFetch(fetchConfig) {
    // Just a proxy to fetchFromAPI, but it has to be a generator.
    const result = yield actions.fetchFromAPI(fetchConfig);
    return result;
  }
};
const selectors = {
  isLoading: state => state.isLoading,
  isQuietLoading: state => state.isQuietLoading,
  getWizardAPIData: (state, slug) => state.apiData[slug] || {},
  getError: state => state.error
};
const store = (0, _data.createReduxStore)(WIZARD_STORE_NAMESPACE, {
  reducer,
  actions,
  selectors,
  controls: {
    FETCH_FROM_API: action => {
      (0, _data.dispatch)(WIZARD_STORE_NAMESPACE).startLoadingData({
        isQuietLoading: Boolean(action.payload.isQuietFetch)
      });
      return (0, _apiFetch.default)(action.payload).then(data => {
        (0, _data.dispatch)(WIZARD_STORE_NAMESPACE).setError(null);
        return data;
      }).catch(error => {
        (0, _data.dispatch)(WIZARD_STORE_NAMESPACE).setError(error);
        return {
          error
        };
      }).finally(result => {
        (0, _data.dispatch)(WIZARD_STORE_NAMESPACE).finishLoadingData();
        return result;
      });
    }
  },
  resolvers: {
    *getWizardAPIData(slug) {
      if (slug) {
        const data = yield actions.fetchFromAPI({
          path: `/newspack/v1/wizard/${slug}`
        });
        return actions.setAPIDataForWizard({
          slug,
          data
        });
      }
      return actions.finishLoadingData();
    }
  }
});
var _default = () => (0, _data.register)(store);
exports.default = _default;